import sequelizeConnection from "../seqConnection";import { DataTypes, Model, Optional } from "sequelize";

interface CheckInAttributes {
  id: number;
  checkin: Date;
  checkout: Date;
  clientID: number;
  boatID: number;
  destination: string;
  note: string;
}
export interface CheckInAttributesInput
  extends Optional<CheckInAttributes, "id"> {}
export interface CheckInAttributesOutput extends Required<CheckInAttributes> {}
class CheckIn extends Model<CheckInAttributes> implements CheckInAttributes {
  public id!: number;
  public checkin: Date;
  public checkout: Date;
  public clientID: number;
  public boatID: number;
  public destination: string;
  public note: string;

  public readonly createdAt!: Date;
  public readonly updatedAt!: Date;
}
CheckIn.init(
  {
    id: {
      type: DataTypes.INTEGER,
      primaryKey: true,
    },
    checkin: {
      type: DataTypes.DATE,
      allowNull: false,
    },
    checkout: {
      type: DataTypes.DATE,
      allowNull: false,
    },
    clientID: {
      type: DataTypes.INTEGER,
      references: {
        model: "customer",
        key: "id",
      },
    },
    boatID: {
      type: DataTypes.INTEGER,
      references: {
        model: "boat",
        key: "id",
      },
    },
    destination: {
      type: new DataTypes.STRING(),
    },
    note: {
      type: new DataTypes.STRING(),
    },
  },
  {
    tableName: "checkIn",
    sequelize: sequelizeConnection,
  }
);

export const create = async (
  payload: CheckIn
): Promise<CheckInAttributesOutput> => {
  return await CheckIn.create(payload);
};

export const update = async (
  id: number,
  payload: Partial<CheckInAttributesInput>
): Promise<CheckInAttributesOutput> => {
  const checkin = await CheckIn.findByPk(id);
  if (!checkin) {
    throw new Error("not found");
  }
  return await (checkin as CheckIn).update(payload);
};

export const getById = async (id: number): Promise<CheckIn> => {
  const checkin = await CheckIn.findByPk(id);
  if (!checkin) {
    throw new Error("not found");
  }
  return checkin;
};

const deleteById = async (id: number): Promise<boolean> => {
  const deletedCheckInCount = await CheckIn.destroy({
    where: { id },
  });
  return !!deletedCheckInCount;
};
export const getAll = async (): Promise<CheckInAttributesOutput[]> => {
  return CheckIn.findAll();
};
export const CheckInService = {
  create,
  update,
  getById,
  deleteById,
  getAll,
};

export default CheckIn;
