import sequelizeConnection from "../seqConnection";import { DataTypes, Model, Optional } from "sequelize";

interface WorkerAttributes {
  email: string;
  firstName: string;
  lastName: string;
  password: string;
  role: string;
}
export interface WorkerInput extends Optional<WorkerAttributes, "email"> {}
export interface WorkerOutput extends Required<WorkerAttributes> {}
class Worker extends Model<WorkerAttributes> implements WorkerAttributes {
  public email!: string;
  public firstName: string;
  public lastName: string;
  public password: string;
  public role: string;

  public readonly createdAt!: Date;
  public readonly updatedAt!: Date;
}
Worker.init(
  {
    email: {
      type: new DataTypes.STRING(),
      allowNull: false,
      primaryKey: true,
      unique: true,
    },
    firstName: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
    lastName: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
    password: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
    role: {
      type: new DataTypes.STRING(),
      allowNull: false,
    },
  },
  {
    tableName: "worker",
    sequelize: sequelizeConnection,
  }
);

export const create = async (payload: Worker): Promise<WorkerOutput> => {
  return await Worker.create(payload);
};

export const update = async (
  email: string,
  payload: Partial<WorkerInput>
): Promise<WorkerOutput> => {
  const worker = await Worker.findByPk(email);
  if (!worker) {
    throw new Error("not found");
  }
  return await (worker as Worker).update(payload);
};

export const getById = async (email: string): Promise<WorkerInput> => {
  const worker = await Worker.findByPk(email);
  if (!worker) {
    throw new Error("not found");
  }
  return worker;
};

const deleteById = async (email: string): Promise<boolean> => {
  const deletedWorkerCount = await Worker.destroy({
    where: { email },
  });
  return !!deletedWorkerCount;
};
export const getAll = async (): Promise<WorkerOutput[]> => {
  return Worker.findAll();
};
export const WorkerService = {
  create,
  update,
  getById,
  deleteById,
  getAll,
};

export default Worker;
